<!doctype html>
<meta charset="utf-8">
<link rel="shortcut icon" href="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAEAAAABACAMAAACdt4HsAAAAGXRFWHRTb2Z0d2FyZQBBZG9iZSBJbWFnZVJlYWR5ccllPAAAABhQTFRFdHR0UFBQwsLCnZ2dCQkJ7+/vhoaG////U3R9JgAAAAh0Uk5T/////////wDeg71ZAAABrElEQVR42tyXgZKEIAiGwdR7/zc+QDT1FK3m2pmlbdtKPn/AtIUfww5HG5nVBqyb6n8f8FkF0nsmGBCweqftPiB5F//bCq4BgKyOP5A514YRIcISkHsfAcg/bihICXhXAZKBWo4/AzIEEJAV1CoaQCbk3nsArgArBeQP6h9NBfMQLICvFaQQ+Lcnq3MAOyFQazcCkD9GKwRqj5YC3K1CzsEohDnA1yG4g485iQy5XIURwK4CbihYVUEIaCkAowqDcXD5WYDvexZGOYC9+WAawrVn4XIS8dGUpjlgDjv0gEi2VYUFYDkO0AIsH6ZE8DdD8Fh6nwHEzCoUQJgC7BAeKvAaf7AApgJX2VSBUh4BplXQ/Mn+voLeYQbI5wYgPAXYOdgDXFUQOpsBXGf/CAhvKqgbj8wZJoPqCwDyck2GtfHy1k8ksmydG18rVegB3PQPQPzOzwjgdZfZuwfgvgJZ48rceIbQSmgAsqDGeIhDWkSBDpGHZKjeTvjIF9J5+nOSvuiu3KReZRWGci5C9BrEpg1Cq+CkR1VwOJUwVoAjBaG8B2RFdY+dSvH9FWAAf6hAHY0fB5IAAAAASUVORK5CYII=">
<title>Something went wrong</title>
<style>
  .hidden {
    display: none;
  }
  a {
    text-decoration: none;
    font-weight: bold;
    color: inherit;
  }
  .smaller {
    font-size: 12px;
  }
  .sad {
    width: 70px;
  }
  code {
    font-family: 'Monaco', monospace;
    font-size: .8em;
    font-weight: bold;
  }
  html {
    background: linear-gradient(rgba(255,255,255,0.3),rgba(255,255,255,0));
    background-color: #e8ebef;
    background-repeat: no-repeat;
    font-family: -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, Helvetica, Arial, sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol";
    font-size: 15px;
  }
  .aside {
    font-size: 12px;
    float: right;
    color: #666;
    font-weight: bold;
  }
  body {
    margin: 0 auto;
    padding: 0;
  }
  h1 {
    font-size: 30px;
    margin-bottom: 0;
  }
  p, li {
    line-height: 1.4em;
  }
  li {
    margin-bottom: 10px;
  }
  .build-error {
    margin-top: 20px;
  }
  .exception,
  pre {
    background: #fff;
    color: #333;
    font-family: monospace;
    font-size: 14px;
    line-height: 18px;
    margin: 0 40px;
    border-radius: 20px;
    padding: 30px;
    word-break: break-word;
    display: inline-block;
  }
  .footnotes {
    padding: 20px 40px;
    max-width: 650px;
  }
  .traceback {
    margin-top: 10px;
    margin-bottom: 20px;
    font-size: 13px;
    overflow: auto;
    word-wrap: break-word;
  }
  a i {
    display: inline-block;
    height: 8px;
    background-size: contain;
    background-position: center;
    background-repeat: no-repeat;
    margin-left: 5px;
    width: 10px;
    background-image: url(data:image/svg+xml;base64,PD94bWwgdmVyc2lvbj0iMS4wIiBlbmNvZGluZz0iVVRGLTgiIHN0YW5kYWxvbmU9Im5vIj8+Cjxzdmcgd2lkdGg9IjE4cHgiIGhlaWdodD0iMTJweCIgdmlld0JveD0iMCAwIDE4IDEyIiB2ZXJzaW9uPSIxLjEiIHhtbG5zPSJodHRwOi8vd3d3LnczLm9yZy8yMDAwL3N2ZyIgeG1sbnM6eGxpbms9Imh0dHA6Ly93d3cudzMub3JnLzE5OTkveGxpbmsiPgogICAgPCEtLSBHZW5lcmF0b3I6IFNrZXRjaCAzLjYuMSAoMjYzMTMpIC0gaHR0cDovL3d3dy5ib2hlbWlhbmNvZGluZy5jb20vc2tldGNoIC0tPgogICAgPHRpdGxlPlNoYXBlPC90aXRsZT4KICAgIDxkZXNjPkNyZWF0ZWQgd2l0aCBTa2V0Y2guPC9kZXNjPgogICAgPGRlZnM+PC9kZWZzPgogICAgPGcgaWQ9IlBhZ2UtMSIgc3Ryb2tlPSJub25lIiBzdHJva2Utd2lkdGg9IjEiIGZpbGw9Im5vbmUiIGZpbGwtcnVsZT0iZXZlbm9kZCI+CiAgICAgICAgPGcgaWQ9Ik91ci1UZWNobm9sb2d5LShEYXduKSIgdHJhbnNmb3JtPSJ0cmFuc2xhdGUoLTU5MS4wMDAwMDAsIC0xNDA2LjAwMDAwMCkiIGZpbGw9IiMwMDAwMDAiPgogICAgICAgICAgICA8ZyBpZD0iQnVpbGRpbmctb3VyLXBsYW5lIiB0cmFuc2Zvcm09InRyYW5zbGF0ZSgxMzAuMDAwMDAwLCAxMDMwLjAwMDAwMCkiPgogICAgICAgICAgICAgICAgPGcgaWQ9Im1vcmUtKG5hdmlnYXRpb24pIiB0cmFuc2Zvcm09InRyYW5zbGF0ZSg0NzAuMDAwMDAwLCAzODIuMDAwMDAwKSByb3RhdGUoLTI3MC4wMDAwMDApIHRyYW5zbGF0ZSgtNDcwLjAwMDAwMCwgLTM4Mi4wMDAwMDApIHRyYW5zbGF0ZSg0NTIuMDAwMDAwLCAzNjQuMDAwMDAwKSI+CiAgICAgICAgICAgICAgICAgICAgPGcgaWQ9Ikdyb3VwIj4KICAgICAgICAgICAgICAgICAgICAgICAgPHBhdGggZD0iTTE1LDkgTDEyLjg4NSwxMS4xMTUgTDE5Ljc1NSwxOCBMMTIuODg1LDI0Ljg4NSBMMTUsMjcgTDI0LDE4IEwxNSw5IFoiIGlkPSJTaGFwZSI+PC9wYXRoPgogICAgICAgICAgICAgICAgICAgIDwvZz4KICAgICAgICAgICAgICAgIDwvZz4KICAgICAgICAgICAgPC9nPgogICAgICAgIDwvZz4KICAgIDwvZz4KPC9zdmc+);
  }
  hr {
    border: 1px solid #ddd;
    margin: 30px 0;
  }
  header {
    padding: 40px;
    position: relative;
  }
</style>
<body>
<header>
<a class="aside" href="/_grow">Pod: {{pod.root}}</a>

<div class="sad">
<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 36.7 47.42"><polygon points="36.7 47.42 0 47.42 0 11.46 11.46 0 36.7 0 36.7 47.42" style="fill:#ffffff"/><polygon points="11.46 0 11.46 11.46 0 11.46 11.46 0" style="fill:#c4c4c3"/><circle cx="11.75" cy="18.3" r="1.92"/><circle cx="24.95" cy="18.3" r="1.92"/><path d="M26.87,29.08a8.2,8.2,0,0,0-8-5.55,8.21,8.21,0,0,0-8,5.55" style="fill:none;stroke:#000;stroke-linecap:round;stroke-miterlimit:10;stroke-width:2px"/></svg>
</div>

<h1>Something went wrong</h1>
</header>
<div class="exception">
  {% if build_error %}
    <div>An error occurred building <b>{{exception.controller}}</b></div>
    <div class="build-error">{{build_error}}</div>
  {% else %}
    <span>{{exception}}</span>
  {% endif %}
  {% if template_exception %}
    <hr>
    <div>We think the problem is with <b>line {{template_exception.lineno}}</b> of <b>{{template_exception.filename}}</b>.</div>
  {% endif %}
</div>
<div class="footnotes">
{% if status == 404 %}
  <p>No resource matches the requested path. Here's what you can do:</p>
  <ul>
    <li>If requesting a rendered page, ensure all path configurations in content documents, blueprints, and the podspec are correct.
    <li>If requesting an asset, the file corresponding to this route could not be found. Ensure the file exists or check the directory configuration in the podspec.
  </ul>
  {% if home_url %}
    <p>You can also go to the <a href="{{home_url}}">homepage</a> or <a href="/_grow">see all routes</a>.</p>
  {% else %}
    <p>You can also <a href="/_grow">see all routes</a>.</p>
  {% endif %}
{% else %}
  <p>This page failed to render. Here's what you can do:
  <ul>
    <li>Check the view, document, blueprint, or podspec for syntax errors.
  </ul>
  {% if not is_web_exception %}
  <a href="javascript:document.getElementById('traceback').classList.toggle('hidden')">Show traceback<i></i></a>
  {% endif %}
{% endif %}
</div>

{% if not is_web_exception %}
<pre id="traceback" class="smaller traceback hidden">{{traceback}}</pre>
{% endif %}
