<!DOCTYPE html>
<html>
  <head>
    <meta charset="utf-8">
    <title>{{pod.root}}</title>
    <link rel="shortcut icon" href="/_grow/assets/favicon.png">
    <link rel="stylesheet" href="/_grow/ui/css/admin.min.css">
  </head>
  <body>
    <header>
      <p class="subtitle">Pod: {{pod.root}}</p>
      <nav>
        <a href="/_grow/routes/concrete/">Routes</a>
        <a href="/_grow/routes/abstract/">Abstract Routes</a>
      </nav>
    </header>
    <main>
      {% block main %}
        {% if partials %}
          {% for field in partials %}
            {% if not field.partial %}
              {% continue %}
            {% endif %}

            {# Render the partial with the values in {{partial}}. #}
            {% set partial_filename = '/partials/' ~ field.partial ~ '/' ~ field.partial ~ '.html' %}
            {% with partial = field %}
              {% include partial_filename with context %}
            {% endwith %}
          {% endfor %}
        {% endif %}
      {% endblock %}
    </main>
    <footer>
      <p>
        Visit <a href="https://grow.dev">grow.dev</a> to learn more about Grow
        and visit the <a href="https://grow.dev/docs">documentation</a>.
      </p>
    </footer>
  </body>
