{% extends "views/base.html" %}
{% set title = "Translation Catalogs" %}

{% block main %}

{% if pod.catalogs %}
  <h3>{{pod.catalogs|length}} translation catalogs</h3>
  <table>
    <thead>
      <tr>
        <th>Catalog</th>
        <th>Untranslated</th>
        <th>Total</th>
      </tr>
    </thead>
    {% for catalog in pod.catalogs %}
    <tr>
      <td>
        <a href="/_grow/translations/{{catalog.locale}}">{{catalog}}</a>
      </td>
      <td>
        {{catalog.list_untranslated()|length}}
      <td>
        {{catalog|length}}
      </td>
    </tr>
    {% endfor %}
  </table>
{% else %}
  <p>No translations.</p>
{% endif %}

{% endblock %}
